//
//  Lynkeos
//  $Id:$
//
//  Created by Jean-Etienne LAMIAUD on 15/01/2014.
//  Copyright 2014-2020 Jean-Etienne LAMIAUD. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

#import <Cocoa/Cocoa.h>

#include "MyImageStacker.h"

/*!
 * @abstract "Calibration" strategy stacker, ie: stack full frame, mean
 */
@interface MyImageStacker_Calibration : NSObject <MyImageStackerModeStrategy>
{
@private
   MyImageStackerParameters* _params;      //!< Stacking parameters
   LynkeosImageBuffer*   _stack;       //!< Stack of calibration images
}
@end
